@@ -1,18 +1,19 @@
 //! `copy` subcommand
 
 use crate::{
+    Application, RUSTIC_APP, RusticConfig,
     commands::init::init_password,
     helpers::table_with_titles,
-    repository::{CliIndexedRepo, CliRepo},
-    status_err, Application, RusticConfig, RUSTIC_APP,
+    repository::{CliIndexedRepo, CliRepo, get_filtered_snapshots},
+    status_err,
 };
-use abscissa_core::{config::Override, Command, FrameworkError, Runnable, Shutdown};
-use anyhow::{bail, Result};
+use abscissa_core::{Command, FrameworkError, Runnable, Shutdown, config::Override};
+use anyhow::{Result, bail};
 use conflate::Merge;
-use log::{error, info, log, Level};
+use log::{Level, error, info, log};
 use serde::{Deserialize, Serialize};
 
-use rustic_core::{repofile::SnapshotFile, CopySnapshot, Id, KeyOptions};
+use rustic_core::{CopySnapshot, Id, KeyOptions, repofile::SnapshotFile};
 
 /// `copy` subcommand
 #[derive(clap::Parser, Command, Default, Clone, Debug, Serialize, Deserialize, Merge)]
@@ -58,7 +59,9 @@ impl Runnable for CopyCmd {
     fn run(&self) {
         let config = RUSTIC_APP.config();
         if config.copy.targets.is_empty() {
-            status_err!("No target given. Please specify at least 1 target either in the profile or using --target!");
+            status_err!(
+                "No target given. Please specify at least 1 target either in the profile or using --target!"
+            );
             RUSTIC_APP.shutdown(Shutdown::Crash);
         }
         if let Err(err) = config.repository.run_indexed(|repo| self.inner_run(repo)) {
@@ -72,7 +75,7 @@ impl CopyCmd {
     fn inner_run(&self, repo: CliIndexedRepo) -> Result<()> {
         let config = RUSTIC_APP.config();
         let mut snapshots = if self.ids.is_empty() {
-            repo.get_matching_snapshots(|sn| config.snapshot_filter.matches(sn))?
+            get_filtered_snapshots(&repo)?
         } else {
             repo.get_snapshots(&self.ids)?
         };
@@ -85,7 +88,7 @@ impl CopyCmd {
             target_config.merge_profile(target, &mut merge_logs, Level::Error)?;
             // display logs from merging
             for (level, merge_log) in merge_logs {
-                log!(level, "{}", merge_log);
+                log!(level, "{merge_log}");
             }
             let target_opt = &target_config.repository;
             if let Err(err) =
@@ -117,8 +120,10 @@ impl CopyCmd {
             target_repo.open()?
         };
 
-        if repo.config().poly()? != target_repo.config().poly()? {
-            bail!("cannot copy to repository with different chunker parameter (re-chunking not implemented)!");
+        if !repo.config().has_same_chunker(target_repo.config()) {
+            bail!(
+                "cannot copy to repository with different chunker parameter (re-chunking not implemented)!"
+            );
         }
 
         let snaps = target_repo.relevant_copy_snapshots(
