@@ -1,13 +1,12 @@
 //! `restore` subcommand
 
 use crate::{
-    commands::open_repository_indexed, helpers::bytes_size_to_string, status_err, Application,
-    RUSTIC_APP,
+    Application, RUSTIC_APP, helpers::bytes_size_to_string, repository::CliIndexedRepo, status_err,
 };
 
 use abscissa_core::{Command, Runnable, Shutdown};
 use anyhow::Result;
-use log::info;
+use log::{debug, info};
 
 use rustic_core::{LocalDestination, LsOptions, RestoreOptions};
 
@@ -42,7 +41,11 @@ pub(crate) struct RestoreCmd {
 }
 impl Runnable for RestoreCmd {
     fn run(&self) {
-        if let Err(err) = self.inner_run() {
+        if let Err(err) = RUSTIC_APP
+            .config()
+            .repository
+            .run_indexed(|repo| self.inner_run(repo))
+        {
             status_err!("{}", err);
             RUSTIC_APP.shutdown(Shutdown::Crash);
         };
@@ -50,10 +53,9 @@ impl Runnable for RestoreCmd {
 }
 
 impl RestoreCmd {
-    fn inner_run(&self) -> Result<()> {
+    fn inner_run(&self, repo: CliIndexedRepo) -> Result<()> {
         let config = RUSTIC_APP.config();
         let dry_run = config.global.dry_run;
-        let repo = open_repository_indexed(&config.repository)?;
 
         let node =
             repo.node_from_snapshot_path(&self.snap, |sn| config.snapshot_filter.matches(sn))?;
@@ -92,15 +94,19 @@ impl RestoreCmd {
             info!("all file contents are fine.");
         }
 
-        if dry_run {
+        if dry_run && config.global.dry_run_warmup {
             repo.warm_up(restore_infos.to_packs().into_iter())?;
-        } else {
+        } else if !dry_run && !config.global.dry_run_warmup {
             // save some memory
             let repo = repo.drop_data_from_index();
 
             let ls = repo.ls(&node, &ls_opts)?;
             repo.restore(restore_infos, &self.opts, ls, &dest)?;
             println!("restore done.");
+        } else {
+            debug!(
+                "--dry-run is without warmup, --dry-run --dry-run-warmup also issues the warmup script."
+            );
         }
 
         Ok(())
